@@ -326,9 +326,12 @@ def evalSlice(ctx, slice):
     # import pdb; pdb.set_trace()
     res = evalPart(ctx, slice.p)
     i = 0
-    while i < slice.start:
-        res.next()
-        i += 1
+    try:
+        while i < slice.start:
+            res.next()
+            i += 1
+    except StopIteration:
+        return
     i = 0
     for x in res:
         i += 1
